<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                exclude-result-prefixes="xs">

<xsl:strip-space elements="*" />

<xsl:variable name="targetNamespace" select="/xs:schema/@targetNamespace" />

<xsl:template match="xs:include">
   <xsl:variable name="schema" 
                 select="document(@schemaLocation, .)/xs:schema" />
   <xsl:variable name="ns">
      <xsl:value-of select="$targetNamespace" />
      <xsl:if test="not($targetNamespace)">no namespace</xsl:if>
   </xsl:variable>
   <h1>Included Schema</h1>
   <table>
      <tr>
         <th>schema location</th>
         <td><xsl:value-of select="@schemaLocation" /></td>
      </tr>
      <tr>
         <th>target namespace</th>
         <td><xsl:value-of select="$ns" /></td>
      </tr>
      <xsl:if test="$schema/@version">
         <tr>
            <th>version</th>
            <td><xsl:value-of select="$schema/@version" /></td>
         </tr>
      </xsl:if>
   </table>
   <xsl:apply-templates select="$schema/*[not(self::xs:include or 
                                              self::xs:import or 
                                              self::xs:redefine)]" />
   <xsl:apply-templates select="$schema/xs:include | $schema/xs:redefine" />
   <xsl:apply-templates select="$schema/xs:import" />
</xsl:template>

<xsl:template match="xs:redefine">
   <xsl:variable name="schema" 
                 select="document(@schemaLocation, .)/xs:schema" />
   <xsl:variable name="ns">
      <xsl:value-of select="$targetNamespace" />
      <xsl:if test="not($targetNamespace)">no namespace</xsl:if>
   </xsl:variable>
   <h1>Redefined Schema</h1>
   <table>
      <tr>
         <th>schema location</th>
         <td><xsl:value-of select="@schemaLocation" /></td>
      </tr>
      <tr>
         <th>target namespace</th>
         <td><xsl:value-of select="$ns" /></td>
      </tr>
      <xsl:if test="$schema/@version">
         <tr>
            <th>version</th>
            <td><xsl:value-of select="$schema/@version" /></td>
         </tr>
      </xsl:if>
   </table>
   <xsl:apply-templates select="*" />
   <xsl:apply-templates select="$schema/*[not(self::xs:include or 
                                              self::xs:import or 
                                              self::xs:redefine)]" />
   <xsl:apply-templates select="$schema/xs:include | $schema/xs:redefine" />
   <xsl:apply-templates select="$schema/xs:import" />
</xsl:template>

<xsl:template match="xs:import">
   <xsl:variable name="schema" 
                 select="document(@schemaLocation, .)/xs:schema" />
   <xsl:variable name="ns">
      <xsl:value-of select="@namespace" />
      <xsl:if test="not(@namespace)">no namespace</xsl:if>
   </xsl:variable>
   <h1>Imported Schema</h1>
   <table>
      <xsl:if test="@schemaLocation">
         <tr>
            <th>schema location</th>
            <td><xsl:value-of select="@schemaLocation" /></td>
         </tr>
      </xsl:if>
      <tr>
         <th>target namespace</th>
         <td><xsl:value-of select="$ns" /></td>
      </tr>
      <xsl:if test="$schema/@version">
         <tr>
            <th>version</th>
            <td><xsl:value-of select="$schema/@version" /></td>
         </tr>
      </xsl:if>
   </table>
   <xsl:apply-templates select="$schema/*[not(self::xs:include or 
                                              self::xs:import or 
                                              self::xs:redefine)]" />
   <xsl:apply-templates select="$schema/xs:include | $schema/xs:redefine" />
   <xsl:apply-templates select="$schema/xs:import" />
</xsl:template>

<xsl:key name="typeDefinitions" 
         match="/xs:schema/xs:simpleType" 
         use="concat('{', ../@targetNamespace, '}', @name)" />
<xsl:key name="typeDefinitions" 
         match="/xs:schema/xs:complexType" 
         use="concat('{', ../@targetNamespace, '}', @name)" />
<xsl:key name="typeDefinitions" 
         match="/xs:schema/xs:redefine/xs:simpleType" 
         use="concat('{', ../../@targetNamespace, '}', @name)" />
<xsl:key name="typeDefinitions" 
         match="/xs:schema/xs:redefine/xs:complexType" 
         use="concat('{', ../../@targetNamespace, '}', @name)" />     

<xsl:key name="attributeDeclarations" 
         match="/xs:schema/xs:attribute" 
         use="concat('{', ../@targetNamespace, '}', @name)" />

<xsl:key name="elementDeclarations" 
         match="/xs:schema/xs:element" 
         use="concat('{', ../@targetNamespace, '}', @name)" />

<xsl:key name="attributeGroupDefinitions" 
         match="/xs:schema/xs:attributeGroup" 
         use="concat('{', ../@targetNamespace, '}', @name)" />
<xsl:key name="attributeGroupDefinitions" 
         match="/xs:schema/xs:redefine/xs:attributeGroup" 
         use="concat('{', ../../@targetNamespace, '}', @name)" />

<xsl:key name="modelGroupDefinitions" 
         match="/xs:schema/xs:group" 
         use="concat('{', ../@targetNamespace, '}', @name)" />
<xsl:key name="modelGroupDefinitions" 
         match="/xs:schema/xs:redefine/xs:group" 
         use="concat('{', ../../@targetNamespace, '}', @name)" />

<xsl:key name="notationDeclarations" 
         match="/xs:schema/xs:notation" 
         use="concat('{', ../@targetNamespace, '}', @name)" />

<xsl:template match="xs:schema">
   <xsl:variable name="ns">
      <xsl:value-of select="@targetNamespace" />
      <xsl:if test="not(@targetNamespace)">no namespace</xsl:if>
   </xsl:variable>
   <xsl:variable name="title" select="'XML Schema Documentation'" />
   <html>
      <head>
         <title><xsl:value-of select="$title" /></title>
         <link href="XMLSchema.css" type="text/css" rel="stylesheet" />
      </head>
      <body>
         <h1><xsl:copy-of select="$title" /></h1>
         <table>
            <tr>
               <th>target namespace</th>
               <td><xsl:value-of select="$ns" /></td>
            </tr>
            <xsl:if test="@version">
               <tr>
                  <th>version</th>
                  <td><xsl:value-of select="@version" /></td>
               </tr>
            </xsl:if>
         </table>
         <xsl:apply-templates select="*[not(self::xs:include or self::xs:import or self::xs:redefine)]" />
         <xsl:apply-templates select="xs:include | xs:redefine" />
         <xsl:apply-templates select="xs:import" />
      </body>
   </html>
</xsl:template>

<xsl:template match="xs:schema/xs:annotation">
   <hr />
   <xsl:apply-templates select="xs:documentation" />
</xsl:template>

<xsl:template match="xs:element[@name]">
   <xsl:variable name="schema" select="ancestor::xs:schema" />
   <xsl:variable name="id">
      <xsl:apply-templates select="." mode="id" />
   </xsl:variable>
   <div class="element">
      <h2 id="{$id}">
         <span class="label">(element)</span>
         <xsl:value-of select="@name" />
      </h2>
      <table>
         <xsl:apply-templates select="xs:annotation" />
         <tr>
            <th>target namespace</th>
            <td><xsl:apply-templates select="." mode="targetNamespace" /></td>
         </tr>
         <xsl:apply-templates select="@substitutionGroup" />
         <xsl:if test="@final or $schema/@finalDefault">
            <tr>
               <th>substitution group exclusions</th>
               <td><xsl:apply-templates select="@final | $schema/@finalDefault[not(current()/@final)]" /></td>
            </tr>
         </xsl:if>
         <xsl:if test="@block or $schema/@blockDefault">
            <tr>
               <th>disallowed substitutions</th>
               <td><xsl:apply-templates select="@block | $schema/@blockDefault[not(current()/@block)]" /></td>
            </tr>
         </xsl:if>
         <tr>
            <th>abstract</th>
            <td>
               <xsl:choose>
                  <xsl:when test="@abstract = 'true' or 
                                  @abstract = 1">true</xsl:when>
                  <xsl:otherwise>false</xsl:otherwise>
               </xsl:choose>
            </td>
         </tr>
         <tr>
            <th>nillable</th>
            <td>
               <xsl:choose>
                  <xsl:when test="@nillable = 'true' or 
                                  @nillable = 1">true</xsl:when>
                  <xsl:otherwise>false</xsl:otherwise>
               </xsl:choose>
            </td>
         </tr>
         <xsl:apply-templates select="@fixed | @default" />
         <xsl:apply-templates select="@type" />
      </table>
      <xsl:apply-templates select="xs:complexType" />
   </div>
</xsl:template>

<xsl:template match="xs:simpleType">
   <xsl:variable name="schema" select="ancestor::xs:schema" />
   <xsl:variable name="id">
      <xsl:apply-templates select="." mode="id" />
   </xsl:variable>
   <div class="simpleType">
      <h2 id="{$id}">
         <span class="label">(simple type)</span>
         <xsl:value-of select="@name" />
      </h2>
      <table>
         <xsl:apply-templates select="xs:annotation" />
         <tr>
            <th>target namespace</th>
            <td><xsl:apply-templates select="." mode="targetNamespace" /></td>
         </tr>
         <xsl:if test="@final or $schema/@finalDefault">
            <tr>
               <th>final</th>
               <td><xsl:apply-templates mode="simple" select="@final | $schema/@finalDefault[not(current()/@final)]" /></td>
            </tr>
         </xsl:if>
         <xsl:apply-templates select="*[not(self::xs:annotation)]" />
      </table>
   </div>
</xsl:template>

<xsl:template match="xs:list">
   <tr>
      <th>list</th>
      <td>
         <xsl:apply-templates select="xs:annotation/xs:documentation" />
         <xsl:choose>
            <xsl:when test="@itemType">
               <xsl:apply-templates select="." mode="link">
                  <xsl:with-param name="qname" select="@itemType" />
               </xsl:apply-templates>
            </xsl:when>
            <xsl:otherwise>
               <xsl:apply-templates select="xs:simpleType" />
            </xsl:otherwise>
         </xsl:choose>
      </td>
   </tr>
</xsl:template>

<xsl:template match="xs:union">
   <tr>
      <th>union</th>
      <td>
         <xsl:apply-templates select="xs:annotation/xs:documentation" />
         <ul>
            <xsl:apply-templates select="." mode="types" />
            <xsl:apply-templates select="*[not(self::xs:annotation)]" />
         </ul>
      </td>
   </tr>
</xsl:template>

<xsl:template match="xs:union" mode="types" name="types">
   <xsl:param name="types" select="normalize-space(@memberTypes)" />
   <xsl:variable name="type">
      <xsl:choose>
         <xsl:when test="contains($types, ' ')"><xsl:value-of select="substring-before($types, ' ')" /></xsl:when>
         <xsl:otherwise><xsl:value-of select="$types" /></xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <li>
      <xsl:apply-templates select="." mode="link">
         <xsl:with-param name="qname" select="$type" />
      </xsl:apply-templates>
   </li>
   <xsl:if test="contains($types, ' ')">
      <xsl:call-template name="types">
         <xsl:with-param name="types" select="substring-after($types, ' ')" />
      </xsl:call-template>
   </xsl:if>
</xsl:template>

<xsl:template match="xs:simpleType/xs:restriction">
   <tr>
      <th>base type definition</th>
      <td>
         <xsl:apply-templates select="." mode="link">
            <xsl:with-param name="qname" select="@base" />
         </xsl:apply-templates>
      </td>
   </tr>
   <tr>
      <th>facets</th>
      <td>
         <xsl:apply-templates select="xs:annotation/xs:documentation" />
         <ul>
            <xsl:if test="xs:enumeration">
               <li>
                  <xsl:text>is one of the values:</xsl:text>
                  <ul>
                     <xsl:for-each select="xs:enumeration">
                        <li><code><xsl:value-of select="@value" /></code></li>
                     </xsl:for-each>
                  </ul>
               </li>
            </xsl:if>
            <xsl:if test="xs:pattern">
               <li>
                  <xsl:choose>
                     <xsl:when test="xs:pattern[2]">
                        <xsl:text>matches one of the patterns:</xsl:text>
                        <ul>
                           <xsl:for-each select="xs:pattern">
                              <li><code><xsl:value-of select="@value" /></code></li>
                           </xsl:for-each>
                        </ul>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:text>matches the pattern </xsl:text>
                        <code><xsl:value-of select="xs:pattern/@value" /></code>
                     </xsl:otherwise>
                  </xsl:choose>
               </li>
            </xsl:if>
            <xsl:apply-templates select="*[not(self::xs:annotation or self::xs:enumeration or self::xs:pattern)]" />
         </ul>
      </td>
   </tr>
</xsl:template>

<xsl:template match="xs:minExclusive">
   <li>
      <xsl:text>is greater than </xsl:text>
      <code><xsl:value-of select="@value" /></code>
   </li>
</xsl:template>

<xsl:template match="xs:minInclusive">
   <li>
      <xsl:text>is greater than or equal to </xsl:text>
      <code><xsl:value-of select="@value" /></code>
   </li>
</xsl:template>
<xsl:template match="xs:maxExclusive">
   <li>
      <xsl:text>is less than </xsl:text>
      <code><xsl:value-of select="@value" /></code>
   </li>
</xsl:template>
<xsl:template match="xs:maxInclusive">
   <li>
      <xsl:text>is less than or equal to </xsl:text>
      <code><xsl:value-of select="@value" /></code>
   </li>
</xsl:template>

<xsl:template match="xs:complexType">
   <xsl:variable name="schema" select="ancestor::xs:schema" />
   <xsl:variable name="id">
      <xsl:apply-templates select="." mode="id" />
   </xsl:variable>
   <div class="complexType">
      <h2 id="{$id}">
         <span class="label">(complex type)</span>
         <xsl:value-of select="@name" />
      </h2>
      <table>
         <xsl:apply-templates select="xs:annotation" />
         <tr>
            <th>target namespace</th>
            <td><xsl:apply-templates select="." mode="targetNamespace" /></td>
         </tr>
         <tr>
            <th>base type definition</th>
            <td>
               <xsl:choose>
                  <xsl:when test="xs:simpleContent | xs:complexContent">
                     <xsl:variable name="derivation" select="(xs:simpleContent | xs:complexContent)/*[not(self::xs:annotation)]" />
                     <xsl:apply-templates select="$derivation" mode="link">
                        <xsl:with-param name="qname" select="$derivation/@base" />
                     </xsl:apply-templates>
                  </xsl:when>
                  <xsl:otherwise>ur-type definition</xsl:otherwise>
               </xsl:choose>
            </td>
         </tr>
         <tr>
            <th>derivation method</th>
            <td>
               <xsl:choose>
                  <xsl:when test="xs:simpleContent | xs:complexContent">
                     <xsl:value-of select="local-name((xs:simpleContent | xs:complexContent)/*[not(self::xs:annotation)])" />
                  </xsl:when>
                  <xsl:otherwise>restriction</xsl:otherwise>
               </xsl:choose>
            </td>
         </tr>
         <xsl:if test="@final or $schema/@finalDefault">
            <tr>
               <th>final</th>
               <td><xsl:apply-templates select="@final | $schema/@finalDefault[not(current()/@final)]" /></td>
            </tr>
         </xsl:if>
         <tr>
            <th>abstract</th>
            <td>
               <xsl:choose>
                  <xsl:when test="@abstract = 'true' or 
                                  @abstract = 1">true</xsl:when>
                  <xsl:otherwise>false</xsl:otherwise>
               </xsl:choose>
            </td>
         </tr>
         <xsl:if test="@block or $schema/@blockDefault">
            <tr>
               <th>prohibited substitutions</th>
               <td><xsl:apply-templates select="@block | $schema/@blockDefault[not(current()/@block)]" /></td>
            </tr>
         </xsl:if>
         <xsl:apply-templates select="." mode="attributes" />
         <xsl:apply-templates select="." mode="content" />
      </table>
      <xsl:apply-templates select="*[not(self::xs:annotation)]" />
   </div>
</xsl:template>

<xsl:template match="xs:complexType" mode="content">
   <tr>
      <th>content</th>
      <td>
         <xsl:variable name="model" select="xs:complexContent | xs:all | xs:sequence | xs:choice | xs:group" />
         <xsl:choose>
            <xsl:when test="xs:simpleContent">
               <xsl:variable name="type" select="xs:simpleContent/*[self::xs:restriction or self::xs:extension]" />
               <xsl:apply-templates select="$type" mode="link">
                  <xsl:with-param name="qname" select="$type/@base" />
               </xsl:apply-templates>
            </xsl:when>
            <xsl:when test="$model">
               <xsl:apply-templates select="$model" mode="model" />
            </xsl:when>
            <xsl:otherwise>empty</xsl:otherwise>
         </xsl:choose>
      </td>
   </tr>
</xsl:template>

<xsl:template match="xs:extension" mode="model" />

<xsl:template match="xs:extension[xs:sequence | xs:choice | xs:group]" mode="model">
   <xsl:text>as </xsl:text>
   <xsl:apply-templates select="." mode="link">
      <xsl:with-param name="qname" select="@base" />
   </xsl:apply-templates>
   <xsl:text>plus </xsl:text>
</xsl:template>

<xsl:template match="xs:restriction" mode="model">empty</xsl:template>

<xsl:template match="xs:restriction[xs:sequence | xs:choice | xs:group]" mode="model">
   <xsl:apply-templates select="xs:sequence | xs:choice | xs:group" />
</xsl:template>

<xsl:template match="*" mode="occurs">
   <xsl:if test="@minOccurs or @maxOccurs">
      <xsl:text>{</xsl:text>
      <xsl:value-of select="@minOccurs" />
      <xsl:if test="not(@minOccurs)">1</xsl:if>
      <xsl:text>,</xsl:text>
      <xsl:value-of select="@maxOccurs" />
      <xsl:if test="not(@maxOccurs)">1</xsl:if>
      <xsl:text>} </xsl:text>
   </xsl:if>
</xsl:template>

<xsl:template match="xs:all | xs:sequence | xs:choice" mode="model">
   <div class="{local-name()}">
      <span class="label">(<xsl:value-of select="local-name()" />)</span>
      <xsl:apply-templates select="." mode="occurs" />
      <xsl:apply-templates select="xs:annotation" />
      <ul>
         <xsl:for-each select="*[not(self::xs:annotation)]">
            <li><xsl:apply-templates select="." mode="model" /></li>
         </xsl:for-each>
      </ul>
   </div>
</xsl:template>

<xsl:template match="xs:group" mode="model">
   <span class="label">(group)</span>
   <xsl:apply-templates select="." mode="occurs" />
   <xsl:apply-templates select="." mode="link">
      <xsl:with-param name="qname" select="@ref" />
      <xsl:with-param name="type" select="'modelGroupDefinitions'" />
   </xsl:apply-templates>
</xsl:template>

<xsl:template match="xs:any" mode="model">
   <xsl:apply-templates select="." mode="occurs" />
   <xsl:text>elements in </xsl:text>
   <xsl:apply-templates select="." mode="wildcard" />
</xsl:template>

<xsl:template match="xs:element" mode="model">
   <span class="label">(element)</span>
   <xsl:apply-templates select="." mode="occurs" />
   <xsl:choose>
      <xsl:when test="@ref">
         <xsl:apply-templates select="." mode="link">
            <xsl:with-param name="qname" select="@ref" />
            <xsl:with-param name="type" select="'elementDeclarations'" />
         </xsl:apply-templates>
      </xsl:when>
      <xsl:otherwise>
         <a href="#{generate-id()}">
            <xsl:value-of select="@name" />
         </a>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:complexType" mode="attributes">
   <tr>
      <th>attributes</th>
      <td>
         <xsl:variable name="attributes">
            <xsl:apply-templates mode="attributes" />
         </xsl:variable>
         <xsl:choose>
            <xsl:when test="string($attributes)">
               <ul>
                  <xsl:copy-of select="$attributes" />
               </ul>
            </xsl:when>
            <xsl:otherwise>none</xsl:otherwise>
         </xsl:choose>
      </td>
   </tr>
</xsl:template>

<xsl:template match="xs:extension" mode="attributes" />

<xsl:template match="xs:extension[xs:attribute | xs:attributeGroup | xs:anyAttribute]" mode="attributes">
   <xsl:text>as </xsl:text>
   <xsl:apply-templates select="." mode="link">
      <xsl:with-param name="qname" select="@base" />
   </xsl:apply-templates>
   <xsl:text>plus </xsl:text>
</xsl:template>

<xsl:template match="xs:restriction" mode="attributes" />

<xsl:template match="xs:restriction[xs:attribute | xs:attributeGroup | xs:anyAttribute]" mode="attributes">
   <xsl:text>as </xsl:text>
   <xsl:apply-templates select="." mode="link">
      <xsl:with-param name="qname" select="@base" />
   </xsl:apply-templates>
   <xsl:text>with the following exceptions </xsl:text>
   <xsl:apply-templates select="xs:attribute | xs:attributeGroup | xs:anyAttribute" />
</xsl:template>

<xsl:template match="xs:attribute" mode="attributes">
   <xsl:variable name="use">
      <xsl:choose>
         <xsl:when test="@use = 'required'">required</xsl:when>
         <xsl:when test="@use = 'prohibited'">prohibited</xsl:when>
         <xsl:otherwise>optional</xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <li class="{$use}">
      <xsl:choose>
         <xsl:when test="@ref">
            <xsl:apply-templates select="." mode="link">
               <xsl:with-param name="qname" select="@ref" />
               <xsl:with-param name="type" select="'attributeDeclarations'" />
            </xsl:apply-templates>
         </xsl:when>
         <xsl:otherwise>   
            <a href="#{generate-id()}">
               <xsl:value-of select="@name" />
            </a>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:if test="@fixed or @default">
         <xsl:text> = </xsl:text>
         <code class="{name(@fixed | @default)}">
            <xsl:value-of select="@fixed | @default" />
         </code>
      </xsl:if>
   </li>
</xsl:template>

<xsl:template match="xs:attributeGroup" mode="attributes">
   <li class="group">
      <span class="label">(attribute group)</span>
      <xsl:apply-templates select="." mode="link">
         <xsl:with-param name="qname" select="@ref" />
         <xsl:with-param name="type" select="'attributeGroupDefinitions'" />
      </xsl:apply-templates>
   </li>
</xsl:template>

<xsl:template match="xs:anyAttribute" mode="attributes">
   <li class="wildcard">
      <xsl:text>attributes in </xsl:text>
      <xsl:apply-templates select="." mode="wildcard" />
   </li>
</xsl:template>

<xsl:template match="*" mode="attributes" />

<xsl:template match="*" mode="wildcard">
   <xsl:variable name="namespace" select="normalize-space(@namespace)" />
   <xsl:choose>
      <xsl:when test="$namespace = '##any'">any namespace</xsl:when>
      <xsl:when test="$namespace = '##other'">any other namespace</xsl:when>
      <xsl:otherwise>
         <ul>
            <xsl:call-template name="wildcard">
               <xsl:with-param name="namespace" select="$namespace" />
            </xsl:call-template>
         </ul>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template name="wildcard">
   <xsl:param name="namespace" />
   <xsl:variable name="ns">
      <xsl:choose>
         <xsl:when test="contains($namespace, ' ')"><xsl:value-of select="substring-before($namespace, ' ')" /></xsl:when>
         <xsl:otherwise><xsl:value-of select="$namespace" /></xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="$ns = '##targetNamespace'">the target namespace</xsl:when>
      <xsl:when test="$ns = '##local'">no namespace</xsl:when>
      <xsl:otherwise>the namespace <code><xsl:value-of select="$ns" /></code></xsl:otherwise>
   </xsl:choose>
   <xsl:if test="contains($namespace, ' ')">
      <xsl:text>, </xsl:text>
      <xsl:call-template name="wildcard">
         <xsl:with-param name="namespace" select="substring-after($namespace, ' ')" />
      </xsl:call-template>
   </xsl:if>
</xsl:template>

<xsl:template match="xs:annotation">
   <xsl:if test="xs:documentation">
      <tr>
         <th>documentation</th>
         <td><xsl:apply-templates select="xs:documentation" /></td>
      </tr>
   </xsl:if>
</xsl:template>

<xsl:template match="xs:documentation">
   <p>
      <xsl:apply-templates />
   </p>
</xsl:template>

<xsl:template match="*" mode="id">
   <xsl:value-of select="generate-id()" />
</xsl:template>

<xsl:template match="xs:element" mode="targetNamespace">
   <xsl:choose>
      <xsl:when test="$targetNamespace and 
                      (parent::xs:schema or
                       @form = 'qualified' or
                       ancestor::xs:schema/@elementFormDefault = 'qualified')">
         <xsl:value-of select="$targetNamespace" />
      </xsl:when>
      <xsl:otherwise>no namespace</xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:attribute" mode="targetNamespace">
   <xsl:choose>
      <xsl:when test="$targetNamespace and 
                      (parent::xs:schema or
                       @form = 'qualified' or
                       ancestor::xs:schema/@attributeFormDefault = 'qualified')">
         <xsl:value-of select="$targetNamespace" />
      </xsl:when>
      <xsl:otherwise>no namespace</xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="*" mode="targetNamespace">
   <xsl:choose>
      <xsl:when test="$targetNamespace">
         <xsl:value-of select="$targetNamespace" />
      </xsl:when>
      <xsl:otherwise>no namespace</xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="*" mode="resolve">
   <xsl:param name="qname" />
   <xsl:variable name="ns" select="namespace::*[name() = substring-before($qname, ':')]" />
   <xsl:variable name="local">
      <xsl:choose>
         <xsl:when test="contains($qname, ':')"><xsl:value-of select="substring-after($qname, ':')" /></xsl:when>
         <xsl:otherwise><xsl:value-of select="$qname" /></xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <xsl:text />{<xsl:value-of select="$ns" />}<xsl:text />
   <xsl:value-of select="$local" />
</xsl:template>

<xsl:template match="@substitutionGroup">
   <tr>
      <th>substitution group affiliation</th>
      <td>
         <xsl:apply-templates select=".." mode="link">
            <xsl:with-param name="qname" select="." />
            <xsl:with-param name="type" select="'elementDeclarations'" />
         </xsl:apply-templates>
      </td>
   </tr>
</xsl:template>

<xs:finalValues>
   <xs:value>extension</xs:value>
   <xs:value>restriction</xs:value>
</xs:finalValues>

<xsl:variable name="finalValues" select="document('')/*/xs:finalValues/xs:value" />

<xsl:template match="@final | @finalDefault">
   <xsl:variable name="final" select="concat(' ', normalize-space(), ' ')" />
   <xsl:call-template name="list">
      <xsl:with-param name="items" select="$finalValues[$final = ' ##all ' or
                                                        contains($final, concat(' ', ., ' '))]" />
   </xsl:call-template>
</xsl:template>

<xs:simpleFinalValues>
   <xs:value>restriction</xs:value>
   <xs:value>list</xs:value>
   <xs:value>union</xs:value>
</xs:simpleFinalValues>

<xsl:variable name="simpleFinalValues" select="document('')/*/xs:simpleFinalValues/xs:value" />

<xsl:template match="@final | @finalDefault" mode="simple">
   <xsl:variable name="final" select="concat(' ', normalize-space(), ' ')" />
   <xsl:call-template name="list">
      <xsl:with-param name="items" select="$simpleFinalValues[$final = ' ##all ' or
                                                        contains($final, concat(' ', ., ' '))]" />
   </xsl:call-template>
</xsl:template>

<xs:blockValues>
   <xs:value>substitution</xs:value>
   <xs:value>extension</xs:value>
   <xs:value>restriction</xs:value>
</xs:blockValues>

<xsl:variable name="blockValues" select="document('')/*/xs:blockValues/xs:value" />

<xsl:template match="@block | @blockDefault">
   <xsl:variable name="block" select="concat(' ', normalize-space(), ' ')" />
   <xsl:call-template name="list">
      <xsl:with-param name="items" select="$blockValues[$block = ' ##all ' or
                                                        contains($block, concat(' ', ., ' '))]" />
   </xsl:call-template>
</xsl:template>

<xsl:template name="list">
  <xsl:param name="items" />
  <xsl:for-each select="$items">
    <xsl:value-of select="." />
    <xsl:if test="position() != last()">, </xsl:if>
  </xsl:for-each>
</xsl:template>

<xsl:template match="@fixed | @default">
   <tr>
      <th><xsl:value-of select="name()" /> value</th>
      <td><xsl:value-of select="." /></td>
   </tr>
</xsl:template>

<xsl:template match="@type">
   <tr>
      <th>type definition</th>
      <td>
         <xsl:apply-templates select=".." mode="link">
            <xsl:with-param name="qname" select="." />
         </xsl:apply-templates>
      </td>
   </tr>
</xsl:template>

<xsl:template match="xs:redefine/xs:complexType/*/xs:extension | 
                     xs:redefine/xs:complexType/*/xs:restriction |
                     xs:redefine/xs:simpleType/xs:restriction |
                     xs:redefine/xs:group//xs:group[@ref = ancestor::xs:group[parent::xs:redefine]/@name or
                                                    substring-after(@ref, ':') = ancestor::xs:group[parent::xs:redefine]/@name] |
                     xs:redefine/xs:attributeGroup/xs:attributeGroup[@ref = ../@name or 
                                                                     substring-after(@ref, ':') = ../@name]" mode="link">
   <xsl:param name="qname" />
   <xsl:param name="type" select="'typeDefinitions'" />
   <xsl:variable name="name">
      <xsl:apply-templates select="." mode="resolve">
         <xsl:with-param name="qname" select="$qname" />
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:variable name="id">
      <xsl:apply-templates select="ancestor::xs:redefine" mode="getID">
         <xsl:with-param name="ns" select="substring-before(substring($name, 2), '}')" />
         <xsl:with-param name="localName" select="substring-after($name, '}')" />
         <xsl:with-param name="type" select="$type" />
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="string($id)">
         <a href="#{$id}">
            <xsl:value-of select="$qname" />
         </a>
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$qname" /></xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="*" mode="link">
   <xsl:param name="qname" />
   <xsl:param name="type" select="'typeDefinitions'" />
   <xsl:variable name="name">
      <xsl:apply-templates select="." mode="resolve">
         <xsl:with-param name="qname" select="$qname" />
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:variable name="id">
      <xsl:apply-templates select="$schema" mode="getID">
         <xsl:with-param name="ns" select="substring-before(substring($name, 2), '}')" />
         <xsl:with-param name="localName" select="substring-after($name, '}')" />
         <xsl:with-param name="type" select="$type" />
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="string($id)">
         <a href="#{$id}">
            <xsl:value-of select="$qname" />
         </a>
      </xsl:when>
      <xsl:otherwise><xsl:value-of select="$qname" /></xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:variable name="schema" select="/xs:schema" />

<xsl:template match="xs:schema" mode="getID">
   <xsl:param name="ns" />
   <xsl:param name="localName" />
   <xsl:param name="type" select="'typeDefinitions'" />
   <xsl:param name="targetNS" select="@targetNamespace" />
   <xsl:variable name="id">
      <xsl:if test="$targetNS = $ns">
         <xsl:value-of select="generate-id(key($type, concat('{', @targetNamespace, '}', $localName)))" />
      </xsl:if>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="string($id)">
         <xsl:value-of select="$id" />
      </xsl:when>
      <xsl:otherwise>
         <xsl:apply-templates select="(xs:include | xs:redefine | xs:import)[1]" mode="getID">
            <xsl:with-param name="ns" select="$ns" />
            <xsl:with-param name="localName" select="$localName" />
            <xsl:with-param name="type" select="$type" />
            <xsl:with-param name="targetNS" select="$targetNS" />
         </xsl:apply-templates>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:import" mode="getID">
   <xsl:param name="ns" />
   <xsl:param name="localName" />
   <xsl:param name="type" select="'typeDefinitions'" />
   <xsl:param name="targetNS" select="@namespace" />
   <xsl:variable name="id">
      <xsl:apply-templates select="document(@schemaLocation, .)/xs:schema" mode="getID">
         <xsl:with-param name="ns" select="$ns" />
         <xsl:with-param name="localName" select="$localName" />
         <xsl:with-param name="type" select="$type" />
         <xsl:with-param name="targetNS" select="@namespace" />
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="string($id)">
         <xsl:value-of select="$id" />
      </xsl:when>
      <xsl:otherwise>
         <xsl:apply-templates select="following-sibling::*[self::xs:include or self::xs:redefine or self::import][1]" mode="getID">
            <xsl:with-param name="ns" select="$ns" />
            <xsl:with-param name="localName" select="$localName" />
            <xsl:with-param name="type" select="$type" />
            <xsl:with-param name="targetNS" select="$targetNS" />
         </xsl:apply-templates>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:include | xs:redefine" mode="getID">
   <xsl:param name="ns" />
   <xsl:param name="localName" />
   <xsl:param name="type" select="'typeDefinitions'" />
   <xsl:param name="targetNS" select="$targetNamespace" />
   <xsl:variable name="id">
      <xsl:apply-templates select="document(@schemaLocation, .)/xs:schema" mode="getID">
         <xsl:with-param name="ns" select="$ns" />
         <xsl:with-param name="localName" select="$localName" />
         <xsl:with-param name="type" select="$type" />
         <xsl:with-param name="targetNS" select="$targetNS" />
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="string($id)">
         <xsl:value-of select="$id" />
      </xsl:when>
      <xsl:otherwise>
         <xsl:apply-templates select="following-sibling::*[self::xs:include or self::xs:redefine or self::import][1]" mode="getID">
            <xsl:with-param name="ns" select="$ns" />
            <xsl:with-param name="localName" select="$localName" />
            <xsl:with-param name="type" select="$type" />
            <xsl:with-param name="targetNS" select="$targetNS" />
         </xsl:apply-templates>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

</xsl:stylesheet>