<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns:my="http://www.jenitennison.com/"
                exclude-result-prefixes="xs xsi my">

<xsl:strip-space elements="*" />

<xsl:variable name="instance" select="/" />

<xsl:variable name="noNamespaceSchemaLocation" select="normalize-space($instance/*/@xsi:noNamespaceSchemaLocation)" />
<xsl:variable name="schemaLocation" select="normalize-space($instance/*/@xsi:schemaLocation)" />

<xsl:param name="schemaDoc">
   <xsl:call-template name="schemaLocation">
      <xsl:with-param name="ns" select="namespace-uri($instance/*)" />
   </xsl:call-template>
</xsl:param>
<xsl:variable name="schema" select="document($schemaDoc, $instance)" />

<xsl:template name="schemaLocation">
   <xsl:param name="ns" />
   <xsl:choose>
      <xsl:when test="$ns">
         <xsl:value-of select="substring-before(substring-after(concat(' ', $schemaLocation, ' '), concat(' ', $ns, ' ')), ' ')" />
      </xsl:when>
      <xsl:otherwise>
         <xsl:value-of select="$noNamespaceSchemaLocation" />
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="/">
   <html>
      <head>
         <title>Form from XML Schema</title>
          <link href="XMLSchema.css" type="text/css" rel="stylesheet" />
     </head>
      <body>
         <h1>Form from XML Schema</h1>
         <form action="">
            <table>
               <xsl:variable name="docElement" select="*" />
               <xsl:variable name="docElementQName" select="concat('{', namespace-uri(*), '}', local-name(*))" />
               <xsl:for-each select="$schema">
                  <xsl:apply-templates select="key('elementDeclarations', $docElementQName)">
                     <xsl:with-param name="node" select="$docElement" />
                  </xsl:apply-templates>
               </xsl:for-each>
            </table>
         </form>
      </body>
   </html>
</xsl:template>

<xsl:template match="xs:element">
   <xsl:param name="node" select="/.." />
   <xsl:param name="id" select="@name" />
   <xsl:variable name="typeName">
      <xsl:apply-templates select="." mode="resolve">
         <xsl:with-param name="qname" select="@type" />
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:variable name="type" select="xs:complexType | xs:simpleType |
                                     key('typeDefinitions', $typeName)" />
   <tr>
      <xsl:choose>
         <xsl:when test="$type[self::xs:complexType]">
            <td colspan="2">
               <fieldset>
                  <legend><xsl:apply-templates select="." mode="label" /></legend>
                  <xsl:apply-templates select="$type">
                     <xsl:with-param name="node" select="$node" />
                     <xsl:with-param name="id" select="$id" />
                  </xsl:apply-templates>
               </fieldset>
            </td>
         </xsl:when>
         <xsl:otherwise>
            <th>
               <label for="{$id}">
                  <xsl:apply-templates select="." mode="label" />
               </label>
            </th>
            <xsl:choose>
               <xsl:when test="starts-with($typeName, '{http://www.w3.org/2001/XMLSchema}')">
                  <td>
                     <xsl:call-template name="createInput">
                        <xsl:with-param name="type" select="substring-after($typeName, '{http://www.w3.org/2001/XMLSchema}')" />
                        <xsl:with-param name="value" select="$node" />
                        <xsl:with-param name="id" select="$id" />
                     </xsl:call-template>
                  </td>
               </xsl:when>
               <xsl:otherwise>
                  <td>
                     <xsl:apply-templates select="$type">
                        <xsl:with-param name="node" select="$node" />
                        <xsl:with-param name="id" select="$id" />
                     </xsl:apply-templates>
                     <xsl:if test="not($type)">
                        <xsl:call-template name="createInput">
                           <xsl:with-param name="type" select="'string'" />
                           <xsl:with-param name="value" select="$node" />
                           <xsl:with-param name="id" select="$id" />
                        </xsl:call-template>
                     </xsl:if>
                  </td>
               </xsl:otherwise>
            </xsl:choose>
         </xsl:otherwise>
      </xsl:choose>
   </tr>
</xsl:template>

<xsl:template match="xs:complexType">
   <xsl:param name="node" select="/.." />
   <xsl:param name="id" />
   <table>
      <xsl:apply-templates select="xs:attribute | xs:attributeGroup | xs:anyAttribute">
         <xsl:with-param name="node" select="$node" />
         <xsl:with-param name="id" select="$id" />
      </xsl:apply-templates>
      <xsl:apply-templates select="xs:choice | xs:sequence | xs:group | xs:all" mode="particle">
         <xsl:with-param name="nodes" select="$node/*" />
         <xsl:with-param name="id" select="$id" />
      </xsl:apply-templates>
   </table>
</xsl:template>

<xsl:template match="xs:sequence | xs:choice" mode="particle">
   <xsl:param name="nodes" select="/.." />
   <xsl:param name="id" />
   <xsl:param name="count" select="1" />
   <tr>
      <td colspan="2">
         <fieldset>
            <legend>
               <xsl:apply-templates select="." mode="label" />
               <xsl:if test="$count > 1"> (<xsl:value-of select="$count" />)</xsl:if>
            </legend>
            <table>
               <xsl:apply-templates select="." mode="particleRecurse">
                  <xsl:with-param name="nodes" select="$nodes" />
                  <xsl:with-param name="id" select="$id" />
                  <xsl:with-param name="count" select="$count" />
               </xsl:apply-templates>
            </table>
         </fieldset>
      </td>
   </tr>
</xsl:template>

<xsl:template match="xs:sequence" mode="particleRecurse">
   <xsl:param name="nodes" select="/.." />
   <xsl:param name="count" select="1" />
   <xsl:param name="id" />
   <xsl:param name="particles" select="xs:*[not(self::xs:annotation)]" />
   <xsl:variable name="matches">
      <xsl:apply-templates select="$particles[1]" mode="test">
         <xsl:with-param name="nodes" select="$nodes" />
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="$matches = -1">
         <xsl:message terminate="yes">invalid document</xsl:message>
      </xsl:when>
      <xsl:otherwise>
         <xsl:apply-templates select="$particles[1]" mode="particle">
            <xsl:with-param name="nodes" select="$nodes[position() &lt;= $matches]" />
            <xsl:with-param name="id" select="concat($id, '.sequence', $count)" />
         </xsl:apply-templates>
         <xsl:variable name="rest" select="$nodes[position() > $matches]" />
         <xsl:choose>
            <xsl:when test="$particles[2]">
               <xsl:apply-templates select="." mode="particleRecurse">
                  <xsl:with-param name="nodes" select="$rest" />
                  <xsl:with-param name="particles" select="$particles[position() > 1]" />
                  <xsl:with-param name="id" select="$id" />
               </xsl:apply-templates>
            </xsl:when>
            <xsl:when test="$rest">
               <xsl:apply-templates select="." mode="particle">
                  <xsl:with-param name="nodes" select="$rest" />
                  <xsl:with-param name="count" select="$count + 1" />
                  <xsl:with-param name="id" select="$id" />
               </xsl:apply-templates>
            </xsl:when>
         </xsl:choose>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:choice" mode="particleRecurse">
   <xsl:param name="nodes" select="/.." />
   <xsl:param name="count" select="1" />
   <xsl:param name="id" />
   <xsl:param name="particles" select="xs:*[not(self::xs:annotation)]" />
   <xsl:variable name="matches">
      <xsl:apply-templates select="$particles[1]" mode="test">
         <xsl:with-param name="nodes" select="$nodes" />
      </xsl:apply-templates>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="not($matches) or $matches = -1">
         <xsl:apply-templates select="$particles[1]" mode="particle">
            <xsl:with-param name="id" select="concat($id, '.choice', $count)" />
         </xsl:apply-templates>
         <xsl:choose>
            <xsl:when test="$particles[2]">
               <xsl:apply-templates select="." mode="particleRecurse">
                  <xsl:with-param name="nodes" select="$nodes" />
                  <xsl:with-param name="particles" select="$particles[position() > 1]" />
                  <xsl:with-param name="id" select="$id" />
               </xsl:apply-templates>
            </xsl:when>
            <xsl:otherwise>
               <xsl:message terminate="yes">invalid document</xsl:message>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:when>
      <xsl:when test="number($matches)">
         <xsl:apply-templates select="$particles[1]" mode="particle">
            <xsl:with-param name="nodes" select="$nodes[position() &lt;= $matches]" />
            <xsl:with-param name="id" select="concat($id, '.choice', $count)" />
         </xsl:apply-templates>
         <xsl:for-each select="$particles[position() > 1]">
            <xsl:apply-templates select="." mode="particle">
               <xsl:with-param name="id" select="concat($id, '.choice', $count)" />
            </xsl:apply-templates>
         </xsl:for-each>
         <xsl:variable name="rest" select="$nodes[position() > $matches]" />
         <xsl:if test="$rest">
            <xsl:apply-templates select="." mode="particle">
               <xsl:with-param name="nodes" select="$rest" />
               <xsl:with-param name="count" select="$count + 1" />
               <xsl:with-param name="id" select="$id" />
            </xsl:apply-templates>
         </xsl:if>
      </xsl:when>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:element" mode="particle">
   <xsl:param name="nodes" select="/.." />
   <xsl:param name="id" />
   <xsl:variable name="ele" select="." />
   <xsl:choose>
      <xsl:when test="$nodes">
         <xsl:for-each select="$nodes">
            <xsl:apply-templates select="$ele">
               <xsl:with-param name="node" select="." />
               <xsl:with-param name="id">
                  <xsl:value-of select="$id" />.<xsl:value-of select="$ele/@name" />
                  <xsl:if test="$nodes[2]">
                     <xsl:value-of select="position()" />
                  </xsl:if>
               </xsl:with-param>
            </xsl:apply-templates>
         </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
         <xsl:apply-templates select="$ele">
            <xsl:with-param name="id" select="concat($id, '.', $ele/@name)" />
         </xsl:apply-templates>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:sequence" mode="test">
   <xsl:param name="nodes" select="/.." />
   <xsl:param name="count" select="1" />
   <xsl:param name="particles" select="xs:*[not(self::xs:annotation)]" />
   <xsl:param name="sum" select="0" />
   <xsl:variable name="minOccurs">
      <xsl:value-of select="@minOccurs" />
      <xsl:if test="not(@minOccurs)">1</xsl:if>
   </xsl:variable>
   <xsl:variable name="maxOccurs">
      <xsl:value-of select="@maxOccurs" />
      <xsl:if test="not(@maxOccurs)">1</xsl:if>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="$count > $maxOccurs"><xsl:value-of select="$sum" /></xsl:when>
      <xsl:otherwise>
         <xsl:variable name="matches">
            <xsl:apply-templates select="$particles[1]" mode="test">
               <xsl:with-param name="nodes" select="$nodes" />
            </xsl:apply-templates>
         </xsl:variable>
         <xsl:choose>
            <xsl:when test="$matches = -1">
               <xsl:choose>
                  <xsl:when test="$count &lt;= @minOccurs">-1</xsl:when>
                  <xsl:otherwise><xsl:value-of select="$sum" /></xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <xsl:when test="$particles[2]">
               <xsl:apply-templates select="." mode="test">
                  <xsl:with-param name="nodes" select="$nodes[position() > $matches]" />
                  <xsl:with-param name="count" select="$count" />
                  <xsl:with-param name="particles" select="$particles[position() > 1]" />
                  <xsl:with-param name="sum" select="$sum + $matches" />
               </xsl:apply-templates>
            </xsl:when>
            <xsl:otherwise>
               <xsl:apply-templates select="." mode="test">
                  <xsl:with-param name="nodes" select="$nodes[position() > $matches]" />
                  <xsl:with-param name="count" select="$count + 1" />
                  <xsl:with-param name="sum" select="$sum + $matches" />
               </xsl:apply-templates>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:choice" mode="test">
   <xsl:param name="nodes" select="/.." />
   <xsl:param name="count" select="1" />
   <xsl:param name="particles" select="xs:*[not(self::xs:annotation)]" />
   <xsl:param name="sum" select="0" />
   <xsl:variable name="minOccurs">
      <xsl:value-of select="@minOccurs" />
      <xsl:if test="not(@minOccurs)">1</xsl:if>
   </xsl:variable>
   <xsl:variable name="maxOccurs">
      <xsl:value-of select="@maxOccurs" />
      <xsl:if test="not(@maxOccurs)">1</xsl:if>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="$count > $maxOccurs"><xsl:value-of select="$sum" /></xsl:when>
      <xsl:otherwise>
         <xsl:variable name="matches">
            <xsl:apply-templates select="$particles[1]" mode="test">
               <xsl:with-param name="nodes" select="$nodes" />
            </xsl:apply-templates>
         </xsl:variable>
         <xsl:choose>
            <xsl:when test="$matches = 0 or $matches = -1">
               <xsl:choose>
                  <xsl:when test="$particles[2]">
                     <xsl:apply-templates select="." mode="test">
                        <xsl:with-param name="nodes" select="$nodes" />
                        <xsl:with-param name="count" select="$count" />
                        <xsl:with-param name="particles" select="$particles[position() > 1]" />
                        <xsl:with-param name="sum" select="$sum" />
                     </xsl:apply-templates>
                  </xsl:when>
                  <xsl:when test="$count &lt;= $minOccurs">-1</xsl:when>
                  <xsl:otherwise><xsl:value-of select="$sum" /></xsl:otherwise>
               </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
               <xsl:apply-templates select="." mode="test">
                  <xsl:with-param name="nodes" select="$nodes[position() > $matches]" />
                  <xsl:with-param name="count" select="$count + 1" />
                  <xsl:with-param name="sum" select="$sum + $matches" />
               </xsl:apply-templates>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:element" mode="test">
   <xsl:param name="nodes" select="/.." />
   <xsl:param name="count" select="1" />
   <xsl:param name="sum" select="0" />
   <xsl:variable name="minOccurs">
      <xsl:value-of select="@minOccurs" />
      <xsl:if test="not(@minOccurs)">1</xsl:if>
   </xsl:variable>
   <xsl:variable name="maxOccurs">
      <xsl:value-of select="@maxOccurs" />
      <xsl:if test="not(@maxOccurs)">1</xsl:if>
   </xsl:variable>
   <xsl:choose>
      <xsl:when test="$count &gt; $maxOccurs">
         <xsl:value-of select="$sum" />
      </xsl:when>
      <xsl:otherwise>
         <xsl:variable name="match" select="local-name($nodes[1]) = @name" />
         <xsl:choose>
            <xsl:when test="$match">
               <xsl:apply-templates select="." mode="test">
                  <xsl:with-param name="nodes" select="$nodes[position() > 1]" />
                  <xsl:with-param name="count" select="$count + 1" />
                  <xsl:with-param name="sum" select="$sum + 1" />
               </xsl:apply-templates>
            </xsl:when>
            <xsl:when test="$count &lt; $minOccurs">-1</xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="$sum" />
            </xsl:otherwise>
         </xsl:choose>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:simpleType">
   <xsl:param name="node" />
   <xsl:param name="id" />
   <xsl:apply-templates select="xs:restriction | xs:list | xs:union">
      <xsl:with-param name="node" select="$node" />
      <xsl:with-param name="id" select="$id" />
   </xsl:apply-templates>
</xsl:template>

<xsl:template match="xs:simpleType/xs:restriction">
   <xsl:param name="node" />
   <xsl:param name="id" />
   <xsl:param name="multiple" select="false()" />
   <xsl:choose>
      <xsl:when test="xs:enumeration">
         <select name="{$id}">
            <xsl:if test="$multiple">
               <xsl:attribute name="multiple">multiple</xsl:attribute>
            </xsl:if>
            <xsl:for-each select="xs:enumeration">
               <option value="{@value}">
                  <xsl:if test="contains(concat(' ', $node, ' '), @value)">
                     <xsl:attribute name="selected">selected</xsl:attribute>
                  </xsl:if>
                  <xsl:apply-templates select="." mode="label" />
               </option>
            </xsl:for-each>
         </select>
      </xsl:when>
      <xsl:otherwise>
         <input value="{$node}" name="{$id}">
            <xsl:if test="xs:length or xs:maxLength">
               <xsl:attribute name="maxlength"><xsl:value-of select="(xs:length | xs:maxLength)/@value" /></xsl:attribute>
            </xsl:if>
            <xsl:variable name="facets" select="xs:*[not(self::xs:maxLength)]" />
            <xsl:if test="$facets">
               <xsl:attribute name="onblur">test<xsl:value-of select="generate-id()" />(this.value);</xsl:attribute>
               <script>
                  function test<xsl:value-of select="generate-id()" />(value) {
                     <xsl:text>if (</xsl:text>
                     <xsl:for-each select="$facets">
                        <xsl:apply-templates select="." />
                        <xsl:if test="position() != last()"> || </xsl:if>
                     </xsl:for-each>
                     <xsl:text>)</xsl:text> {
                        alert('invalid value');
                     }
                  }
               </script>
            </xsl:if>
         </input>
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:template match="xs:minInclusive">
   <xsl:text />value &lt; <xsl:value-of select="@value" />
</xsl:template>
<xsl:template match="xs:maxInclusive">
   <xsl:text />value > <xsl:value-of select="@value" />
</xsl:template>
<xsl:template match="xs:minExclusive">
   <xsl:text />value &lt;= <xsl:value-of select="@value" />
</xsl:template>
<xsl:template match="xs:maxExclusive">
   <xsl:text />value >= <xsl:value-of select="@value" />
</xsl:template>
<xsl:template match="xs:length">
   <xsl:text />value.length &lt; <xsl:value-of select="@value" />
</xsl:template>
<xsl:template match="xs:pattern">
   <xsl:text />!matches(value, '<xsl:value-of select="@value" />')<xsl:text />
</xsl:template>

<xsl:template match="*[xs:annotation/xs:appinfo/my:label]" mode="label" priority="1">
   <xsl:value-of select="xs:annotation/xs:appinfo/my:label" />
</xsl:template>

<xsl:template match="*[@name]" mode="label">
   <xsl:value-of select="@name" />
</xsl:template>

<xsl:template match="xs:enumeration" mode="label">
   <xsl:value-of select="@value" />
</xsl:template>

<xsl:template match="*" mode="label">
   <xsl:value-of select="local-name()" />
</xsl:template>

<xsl:template match="*" mode="resolve">
   <xsl:param name="qname" />
   <xsl:variable name="ns" select="namespace::*[name() = substring-before($qname, ':')]" />
   <xsl:variable name="local">
      <xsl:choose>
         <xsl:when test="contains($qname, ':')"><xsl:value-of select="substring-after($qname, ':')" /></xsl:when>
         <xsl:otherwise><xsl:value-of select="$qname" /></xsl:otherwise>
      </xsl:choose>
   </xsl:variable>
   <xsl:text />{<xsl:value-of select="$ns" />}<xsl:text />
   <xsl:value-of select="$local" />
</xsl:template>

<xsl:template name="createInput">
   <xsl:param name="type" />
   <xsl:param name="value" />
   <xsl:param name="id" />
   <xsl:choose>
      <xsl:when test="$type = 'boolean'">
         <input name="{$id}" type="checkbox">
            <xsl:if test="$value = 'true' or $value = '1'">
               <xsl:attribute name="checked">checked</xsl:attribute>
            </xsl:if>
         </input>
      </xsl:when>
      <xsl:otherwise>
         <input name="{$id}" value="{$value}" />
      </xsl:otherwise>
   </xsl:choose>
</xsl:template>

<xsl:key name="typeDefinitions" 
         match="/xs:schema/xs:simpleType" 
         use="concat('{', ../@targetNamespace, '}', @name)" />
<xsl:key name="typeDefinitions" 
         match="/xs:schema/xs:complexType" 
         use="concat('{', ../@targetNamespace, '}', @name)" />
<xsl:key name="typeDefinitions" 
         match="/xs:schema/xs:redefine/xs:simpleType" 
         use="concat('{', ../../@targetNamespace, '}', @name)" />
<xsl:key name="typeDefinitions" 
         match="/xs:schema/xs:redefine/xs:complexType" 
         use="concat('{', ../../@targetNamespace, '}', @name)" />     

<xsl:key name="attributeDeclarations" 
         match="/xs:schema/xs:attribute" 
         use="concat('{', ../@targetNamespace, '}', @name)" />

<xsl:key name="elementDeclarations" 
         match="/xs:schema/xs:element" 
         use="concat('{', ../@targetNamespace, '}', @name)" />

<xsl:key name="attributeGroupDefinitions" 
         match="/xs:schema/xs:attributeGroup" 
         use="concat('{', ../@targetNamespace, '}', @name)" />
<xsl:key name="attributeGroupDefinitions" 
         match="/xs:schema/xs:redefine/xs:attributeGroup" 
         use="concat('{', ../../@targetNamespace, '}', @name)" />

<xsl:key name="modelGroupDefinitions" 
         match="/xs:schema/xs:group" 
         use="concat('{', ../@targetNamespace, '}', @name)" />
<xsl:key name="modelGroupDefinitions" 
         match="/xs:schema/xs:redefine/xs:group" 
         use="concat('{', ../../@targetNamespace, '}', @name)" />

<xsl:key name="notationDeclarations" 
         match="/xs:schema/xs:notation" 
         use="concat('{', ../@targetNamespace, '}', @name)" />

</xsl:stylesheet>