package xercesvalidator;

/**
 * Title:        Xerces Validator
 * Description:  Class used for validating an instance document with a schema
 * Copyright:    Copyright (c) 2001
 * Company:      Defined Systems
 * @author       Jeff Rafter
 * @version      1.0
 */
 
import org.w3c.dom.Document;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;


public class validate 
  implements ErrorHandler {

  private String instance = null;
  private String schema = null;
  private int warnings = 0;
  private int errors = 0;
  private int fatalErrors = 0;

  private org.apache.xerces.parsers.DOMParser parser = 
    new org.apache.xerces.parsers.DOMParser();

  public void setInstance(String s) {
    instance = s;  
  }
  
  public void setSchema(String s) {
    schema = s;  
    System.out.println("Currently you must specify the schema utilizing xsi:schemaLocation or xsi:noNamespaceSchemaLocation - using default");
  }
  
  public boolean doCheckSourceDocument() {
    try { 
  
      try {          
        // Turn validation off     
        parser.setFeature( "http://xml.org/sax/features/validation", false);   
        // Get the document
        System.out.println("");
        System.out.println("Checking source document...");
        parser.parse(instance);
        // Get the document
        Document doc = parser.getDocument();       
        if (doc != null) {
          // Check if the schema is null
          if (schema != null) {
                 
            // Check for xsi:schemaLocation
            
            // Check for xsi:noNamespaceSchemaLocation !! assuming no namespace for now
            doc.getDocumentElement().setAttribute("xsi:noNamespaceSchemaLocation", schema);
            doc.getDocumentElement().setAttribute("xmlns:xsi", org.apache.xerces.validators.schema.SchemaSymbols.URI_XSI);
           

          // Check if they specified the location       
          } else if ((!doc.getDocumentElement().hasAttribute("xsi:schemaLocation"))         
            && (!doc.getDocumentElement().hasAttribute("xsi:noNamespaceSchemaLocation"))) {

            // Check for a namespace decl for the root element
            
              // Slam it for RDDL
              
              // Slam it for a Schema

            // No location - except out
            throw new Exception("[Error] You must specify the schema for " + instance + 
              " by including the attribute xsi:schemaLocation or xsi:noNamespaceSchemaLocation on the root element");
          }
        }
      } catch (SAXException e) {
        System.out.println("");
        System.err.println("Could not activate validation feature");
        return false;
      }
      // Made it through with no exceptions...
      return true;    
          
    } catch (Exception e) {
      System.out.println("");
      System.err.println(e.getMessage());
      return false;
    }   
  }

  public boolean doValidate() {
    try { 
      warnings = 0;
      errors = 0;
      fatalErrors = 0;
      try {          
        // Now reparse with the validation turned on
        parser.setFeature( "http://xml.org/sax/features/validation", true);   
        System.out.println("Validating source document...");
        parser.parse(instance);
      } catch (SAXException e) {
        System.out.println("");
        System.err.println("Could not activate validation feature");
      }
         
      // Return true if we made it this far with no errors
      System.out.println("");
      System.out.println("Complete " + warnings + " warnings " + errors + " errors " + fatalErrors + " fatal errors");
      return ((errors == 0) && (fatalErrors == 0));
          
    } catch (Exception e) {
      System.out.println("");
      System.err.println(e.getMessage());
      return false;
    }   
  } 
  
  public void warning(SAXParseException ex) {
     System.out.println("");
     System.err.println("[Warning] " + ex.getMessage() + 
       " line " + ex.getLineNumber() + " column " + ex.getColumnNumber() + 
       " - " + instance);
     warnings++;  
  }

  public void error(SAXParseException ex) {
     System.out.println("");
     System.err.println("[Error] " + ex.getMessage() + 
       " line " + ex.getLineNumber() + " column " + ex.getColumnNumber() + 
       " - " + instance);
     errors++;  
  }

  public void fatalError(SAXParseException ex) throws SAXException {
     System.out.println("");
     System.err.println("[Fatal Error] " + ex.getMessage() + 
       " line " + ex.getLineNumber() + " column " + ex.getColumnNumber() + 
       " - " + instance);
     
     fatalErrors++;  
     
     throw ex;  
  }

  public validate() {
    // Set the errorHandler  
    parser.setErrorHandler(this);    
  }
  
  public static void main(String[] args) {
    validate validate1 = new validate();
    if (args.length == 0) {
      System.out.println("Usage : java validate <instance> <schema>");
      return;     
    }
    // Set the instance
    if (args.length >= 1) {
      validate1.setInstance(args[0]);   
    }
    // Set the Schema
    if (args.length >= 2) {    
      validate1.setSchema(args[1]);   
    }
    // Validate - currently doCheckSourceDocument does nothing to modify the 
    // validating document... it can be removed to speed this up...
    if (!validate1.doCheckSourceDocument()) return;
    if (!validate1.doValidate()) return;
  }

  
}
